// @ts-ignore
import { dom, node } from 'jsx-pragmatic';
// @ts-ignore
import zoid from 'zoid';

interface Theme {
  primaryColor: string;
  secondaryColor: string;
  fontFamily: string;
  logo: string;
}
interface NotabeneConfig {
  widget: string;
  container: string;
  authToken: string;
  onValidStateChange(): void;
  theme: Theme;
  dictionary: { [key: string]: string };
}

export interface OwnershipProofInput {
  type?: string | null;
  proof?: string | null;
}

export interface TransactionBlockchainInfo {
  destination?: string | null;
}
interface Transaction {
  transactionAsset: string;
  transactionAmount: string;
  beneficiaryAccountNumber: string;
  originatorVASPdid?: string;
  originatorDid?: string;
  transactionBlockchainInfo?: TransactionBlockchainInfo;
  beneficiaryVASPname?: string;
  beneficiaryName?: string;
  beneficiaryDid?: string;
  isNonCustodial?: boolean;
  originatorEqualsBeneficiary?: boolean;
  beneficiaryProof?: OwnershipProofInput;
  isValid?: boolean;
  errors?: string[];
}

export default class Notabene {
  private widget: zoid.ZoidComponent;
  private popup: zoid.ZoidComponent;
  private config: NotabeneConfig;
  private tx: Transaction | undefined;
  private onValidStateChange: (isValid: boolean) => void;
  private widgetRendered: boolean;
  private popupRendered: boolean;
  private theme: Theme;
  private dictionary: { [key: string]: string };

  constructor(config: NotabeneConfig) {
    this.config = config;
    this.theme = this.config.theme;
    this.dictionary = this.config.dictionary;
    this.widget = this.buildWidget();
    this.popup = this.buildPopup();
    this.onValidStateChange = config.onValidStateChange;
    this.widgetRendered = false;
    this.popupRendered = false;
  }

  public async renderWidget(): Promise<void> {
    if (!this.widget) return;
    this.widgetRendered
      ? console.error('Notabene: Widget is already rendered')
      : await this.widget.render(this.config.container);
  }

  public async renderPopup(): Promise<void> {
    if (!this.popup) return;

    this.popupRendered
      ? this.popup.show()
      : await this.popup.render(this.config.container);
  }

  public setTransaction(tx: Transaction) {
    const validProperties = [
      'transactionAsset',
      'transactionAmount',
      'beneficiaryAccountNumber',
      'beneficiaryAccountNumber',
      'originatorVASPdid',
      'originatorDid',
      'transactionBlockchainInfo',
      'beneficiaryVASPname',
      'beneficiaryName',
      'beneficiaryDid',
      'isNonCustodial',
      'originatorEqualsBeneficiary',
      'beneficiaryProof',
      'isValid',
      'errors',
    ];
    Object.keys(tx).forEach((key) => {
      if (!validProperties.includes(key)) {
        throw new Error(`Notabene: Invalid transaction property ${key}`);
      }
    });
    this.tx = tx;
    this.widget.updateProps({
      transaction: tx,
    });
    this.popup.updateProps({
      transaction: tx,
    });
  }

  private getWidgetUrl(): string {
    const url = new URL(this.config.widget);
    url.hash = `authToken=${this.config.authToken}`;
    return url.toString();
  }

  private getAdditionalInfoUrl(): string {
    const url = new URL(this.config.widget);
    url.search = 'popup=true';
    url.hash = `authToken=${this.config.authToken}`;
    return url.toString();
  }

  private buildWidget(): zoid.ZoidComponent {
    if (!this.config.authToken) {
      console.error('Notabene: authToken is required');
      return;
    }

    const builtWidget = zoid.create({
      tag: 'widget',
      url: this.getWidgetUrl(),
      dimensions: {
        width: '440px',
        height: '280px',
      },
      props: {
        transaction: {
          type: 'object',
          required: false,
        },
        onRequireAdditionalInformation: {
          type: 'function',
        },
        onValidStateChange: {
          type: 'function',
        },
        storeTransaction: {
          type: 'function',
        },
        theme: {
          type: 'object',
          required: false,
        },
        dictionary: {
          type: 'object',
          required: false,
        },
      },
    })({
      transaction: {},
      theme: this.theme,
      dictionary: this.dictionary,
      onRequireAdditionalInformation: () => {
        this.renderPopup();
      },
      onValidStateChange: (isValid: boolean) => {
        this.onValidStateChange(isValid);
      },
      storeTransaction: (tx: Transaction) => {
        this.setTransaction(tx);
      },
    });

    builtWidget.event.on(zoid.EVENT.RENDERED, (err: any) => {
      this.widgetRendered = true;
    });

    return builtWidget;
  }

  private buildPopup(): zoid.ZoidComponent {
    if (!this.config.authToken) {
      console.error('Notabene: authToken is required');
      return;
    }

    const builtPopup = zoid.create({
      tag: 'popup',
      url: this.getAdditionalInfoUrl(),
      children: () => {
        return {
          Transaction: this.tx,
        };
      },
      dimensions: {
        width: '608px',
        height: '100%',
      },
      props: {
        transaction: {
          type: 'object',
          required: false,
        },
        storeTransaction: {
          type: 'function',
        },
        close: {
          type: 'function',
        },
        theme: {
          type: 'object',
          required: false,
        },
        dictionary: {
          type: 'object',
          required: false,
        },
      },
      containerTemplate: ({ uid, doc, frame, prerenderFrame }: any) => {
        return node(
          'div',
          { id: uid, class: 'nb-container' },
          node(
            'style',
            null,
            `
            #${uid}.container {
                border: 5px solid red;
            }
            .nb-overlay {
              position: fixed;
              width: 100vw;
              height: 100vh;
              top: 0;
              left: 0;
              display: flex;
              align-items: center;
              justify-content: center;
              background-color: rgba(0,0,0,0.5);
            }
            .nb-popup {
              max-width: 608px;
              width: 100%;
              height: 100%;
            }
            .nb-preloader {
              display: none;
            }
          `
          ),
          node(
            'div',
            { class: 'nb-overlay' },
            node('node', { el: frame, class: 'nb-popup' }),
            node('node', { el: prerenderFrame, class: 'nb-preloader' })
          )
        ).render(dom({ doc }));
      },
    })({
      transaction: this.tx,
      storeTransaction: (tx: Transaction) => {
        this.setTransaction(tx);
        console.debug('Notabene: Transaction stored');
      },
      close: () => {
        this.popup.close();
        console.debug('Notabene: Popup closing');
      },
      theme: this.theme,
      dictionary: this.dictionary,
    });

    builtPopup.event.on(zoid.EVENT.RENDERED, (err: any) => {
      this.popupRendered = true;
    });

    return builtPopup;
  }
}

window.Notabene = Notabene;
